@@ -20,5 +20,22 @@
  *
  */
 
-bool netconfig_ifindex_add(uint32_t ifindex);
-bool netconfig_ifindex_remove(uint32_t ifindex);
+struct netconfig;
+
+enum netconfig_event {
+	NETCONFIG_EVENT_CONNECTED,
+};
+
+typedef void (*netconfig_notify_func_t)(enum netconfig_event event,
+							void *user_data);
+
+bool netconfig_configure(struct netconfig *netconfig,
+				const struct l_settings *active_settings,
+				const uint8_t *mac_address,
+				netconfig_notify_func_t notify,
+				void *user_data);
+bool netconfig_reconfigure(struct netconfig *netconfig);
+bool netconfig_reset(struct netconfig *netconfig);
+
+struct netconfig *netconfig_new(uint32_t ifindex);
+void netconfig_destroy(struct netconfig *netconfig);
